import React, { useState, useEffect } from 'react';
import Info from './Info';
import * as Drawing from '@mindfusion/drawing';
import * as Diagramming from '@mindfusion/diagramming';
import { DiagramView } from '@mindfusion/diagramming-react';

const onDiagramNodeCreated = (sender: Diagramming.Diagram, args: Diagramming.NodeEventArgs) => {
  var node = args.node;
  if ((sender.defaultShape as Diagramming.Shape).id === "Rectangle")
    node.anchorPattern = apat2;
  else
    node.anchorPattern = decision1In3Out;
}

const blue = "#2d3956";
const green = "#5a7444";
const red = "#ce0000";

const apat1 = new Diagramming.AnchorPattern([
  new Diagramming.AnchorPoint(50, 0, true, true),
  new Diagramming.AnchorPoint(100, 50, true, true),
  new Diagramming.AnchorPoint(50, 100, true, true),
  new Diagramming.AnchorPoint(0, 50, true, true)
]);

const apat2 = new Diagramming.AnchorPattern(
  [
    new Diagramming.AnchorPoint(10, 0, true, false, Diagramming.MarkStyle.Circle, green),
    new Diagramming.AnchorPoint(50, 0, true, false, Diagramming.MarkStyle.Circle, blue),
    new Diagramming.AnchorPoint(90, 0, true, false, Diagramming.MarkStyle.Circle, red),
    new Diagramming.AnchorPoint(10, 100, false, true, Diagramming.MarkStyle.Rectangle),
    new Diagramming.AnchorPoint(50, 100, false, true, Diagramming.MarkStyle.Rectangle),
    new Diagramming.AnchorPoint(90, 100, false, true, Diagramming.MarkStyle.Rectangle),
    new Diagramming.AnchorPoint(0, 50, true, true, Diagramming.MarkStyle.X)
  ]);

const decision1In3Out = Diagramming.AnchorPattern.fromId("Decision1In3Out");
const decision2In2Out = Diagramming.AnchorPattern.fromId("Decision2In2Out");

function AnchorPoints() {
  const [diagram] = useState(new Diagramming.Diagram());
  const [shape, setShape] = useState(Diagramming.Shapes.Rectangle());

  useEffect(() => {
    // setup the diagram
    var theme = new Diagramming.Theme();
    var shapeNodeStyle = new Diagramming.Style();
    shapeNodeStyle.brush = ({ type: 'LinearGradientBrush', color1: '#e0e9e9', color2: '#616a7f', angle: 90 });
    shapeNodeStyle.stroke = ("#7F7F7F");
    shapeNodeStyle.textColor = ("#585A5C");
    shapeNodeStyle.fontName = ("Verdana");
    shapeNodeStyle.fontSize = (3);
    shapeNodeStyle.backBrush = ("#e0e9e9");
    shapeNodeStyle.nodeEffects = [];
    shapeNodeStyle.nodeEffects.push(new Diagramming.GlassEffect());
    theme.styles.set("std:ShapeNode", shapeNodeStyle);
    diagram.theme = theme;

    diagram.defaultShape = "Rectangle";

    var pb1 = diagram.factory.createShapeNode(new Drawing.Rect(10, 7, 25, 18));
    pb1.shape = "Ellipse";
    pb1.text = "Start";
    pb1.anchorPattern = apat1;

    var pb2 = diagram.factory.createShapeNode(new Drawing.Rect(20, 78, 25, 18));
    pb2.text = "node 1";
    pb2.anchorPattern = apat2;

    var pb3 = diagram.factory.createShapeNode(new Drawing.Rect(70, 73, 25, 18));
    pb3.text = "node 2";
    pb3.anchorPattern = apat2;

    var pb4 = diagram.factory.createShapeNode(new Drawing.Rect(80, 110, 25, 18));
    pb4.shape = "Ellipse";
    pb4.text = "End";
    pb4.anchorPattern = apat1;

    var decb1 = diagram.factory.createShapeNode(new Drawing.Rect(20, 35, 30, 20));
    decb1.shape = "Decision";
    decb1.text = "check 1";
    decb1.anchorPattern = decision1In3Out;

    var decb2 = diagram.factory.createShapeNode(new Drawing.Rect(70, 30, 30, 20));
    decb2.shape = "Decision";
    decb2.text = "check 2";
    decb2.anchorPattern = decision2In2Out;

    var link = diagram.factory.createDiagramLink(decb1, decb2);
    link.route();
  }, [diagram]);

  const onShapeChange = (e: React.ChangeEvent<HTMLSelectElement>) => {
    setShape(Diagramming.Shape.fromId(e.target.value));
  }

  const props = {
    "linkHeadShapeSize": 2,
    "routeLinks": true,
    "roundedLinks": true,
    "backBrush": "#e0e9e9",
    "defaultShape": shape
  };

  return (
    <div className="App">
      <div className="header">
        <select id="shape" onChange={onShapeChange}>
          <option value="Rectangle">Node</option>
          <option value="Decision">Decision</option>
        </select>
      </div>

      <div className="container">
        <div className="main">
          <DiagramView diagram={diagram} {...props}
            onNodeCreated={(sender, args) => onDiagramNodeCreated(sender, args)} />
        </div>
        <div className="sidebar">
          <h1>About this sample</h1>
          <p>This sample shows how to define custom anchor-point patterns and assign them to diagram nodes.
            There are also several predefined anchor patterns to choose from.
            Draw links between several nodes to get an idea how anchor points work.</p>
          <Info />
        </div>
      </div>

      <div className="footer">
        <p>Copyright 2024 MindFusion LLC.</p>
      </div>
    </div>
  );
}


export default AnchorPoints;
